package cn.aiyangniu.api.common.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * 描述:此类用于取得当前日期相对应的月初，月末，季初，季末，年初，年末，返回值均为String字符串
 * 1、得到当前日期         today()
 * 2、得到当前月份月初      thisMonth()
 * 3、得到当前月份月底      thisMonthEnd()
 * 4、得到当前季度季初      thisSeason()
 * 5、得到当前季度季末      thisSeasonEnd()
 * 6、得到当前年份年初      thisYear()
 * 7、得到当前年份年底      thisYearEnd()
 * 8、判断输入年份是否为闰年 leapYear
 * 注意事项:  日期格式为：xxxx-yy-zz (eg: 2007-12-05)
 *
 * @author Henry Hall
 */
public class DateUtil {
    private static final Logger logger = LoggerFactory.getLogger(DateUtil.class);

    private int x;                  // 日期属性：年
    private int y;                  // 日期属性：月
    private int z;                  // 日期属性：日
    private Calendar localTime;     // 当前日期

    public DateUtil() {
        localTime = Calendar.getInstance();
    }

    /**
     * 功能：得到当前日期 格式为：xxxx-yy-zz (eg: 2007-12-05)
     */
    public String today() {
        String strY, strZ;
        x = localTime.get(Calendar.YEAR);
        y = localTime.get(Calendar.MONTH) + 1;
        z = localTime.get(Calendar.DATE);
        strY = y >= 10 ? String.valueOf(y) : ("0" + y);
        strZ = z >= 10 ? String.valueOf(z) : ("0" + z);
        return x + "-" + strY + "-" + strZ;
    }

    /**
     * 功能：得到当前月的第一天 格式为：xxxx-yy-zz (eg: 2007-12-01)
     */
    public String thisMonth() {
        String strY;
        x = localTime.get(Calendar.YEAR);
        y = localTime.get(Calendar.MONTH) + 1;
        strY = y >= 10 ? String.valueOf(y) : ("0" + y);
        return x + "-" + strY + "-01";
    }

    /**
     * 功能：得到当前月的最后一天 格式为：xxxx-yy-zz (eg: 2007-12-31)
     */
    public String thisMonthEnd() {
        String strY, strZ = null;
        boolean leap;
        x = localTime.get(Calendar.YEAR);
        y = localTime.get(Calendar.MONTH) + 1;
        if(y == 1 || y == 3 || y == 5 || y == 7 || y == 8 || y == 10 || y == 12) {
            strZ = "31";
        }
        if(y == 4 || y == 6 || y == 9 || y == 11) {
            strZ = "30";
        }
        if(y == 2) {
            leap = leapYear(x);
            if(leap) {
                strZ = "29";
            } else {
                strZ = "28";
            }
        }
        strY = y >= 10 ? String.valueOf(y) : ("0" + y);
        return x + "-" + strY + "-" + strZ;
    }

    /**
     * 功能：得到当前季度季初 格式为：xxxx-yy-zz (eg: 2007-10-01)
     */
    public String thisSeason() {
        String dateString = "";
        x = localTime.get(Calendar.YEAR);
        y = localTime.get(Calendar.MONTH) + 1;
        if(y <= 3) {
            dateString = x + "-" + "01" + "-" + "01";
        }
        if(y >= 4 && y <= 6) {
            dateString = x + "-" + "04" + "-" + "01";
        }
        if(y >= 7 && y <= 9) {
            dateString = x + "-" + "07" + "-" + "01";
        }
        if(y >= 10 && y <= 12) {
            dateString = x + "-" + "10" + "-" + "01";
        }
        return dateString;
    }

    /**
     * 功能：得到当前季度季末 格式为：xxxx-yy-zz (eg: 2007-12-31)
     */
    public String thisSeasonEnd() {
        String dateString = "";
        x = localTime.get(Calendar.YEAR);
        y = localTime.get(Calendar.MONTH) + 1;
        if(y <= 3) {
            dateString = x + "-" + "03" + "-" + "31";
        }
        if(y >= 4 && y <= 6) {
            dateString = x + "-" + "06" + "-" + "30";
        }
        if(y >= 7 && y <= 9) {
            dateString = x + "-" + "09" + "-" + "30";
        }
        if(y >= 10 && y <= 12) {
            dateString = x + "-" + "12" + "-" + "31";
        }
        return dateString;
    }

    /**
     * 功能：得到当前年份年初 格式为：xxxx-yy-zz (eg: 2007-01-01)
     */
    public String thisYear() {
        x = localTime.get(Calendar.YEAR);
        return x + "-01" + "-01";
    }

    /**
     * 功能：得到当前年份年底 格式为：xxxx-yy-zz (eg: 2007-12-31)
     */
    public String thisYearEnd() {
        x = localTime.get(Calendar.YEAR);
        return x + "-12" + "-31";
    }

    /**
     * 功能：判断输入年份是否为闰年
     *
     * @param year 年份
     */
    public boolean leapYear(int year) {
        boolean leap;
        if(year % 4 == 0) {
            if(year % 100 == 0) {
                leap = year % 400 == 0;
            } else {
                leap = true;
            }
        } else leap = false;
        return leap;
    }
    /**
     * 获得日期开始时间 格式为：xxxx-yy-zz hh24:mi:ss (eg: 2007-06-22 00:00:00)
     *
     * @param d 日期
     * @return 日期
     */
    public static Date getStartDate(Date d) {
        Calendar c = Calendar.getInstance();
        if (d != null) {
            c.setTime(d);
        }

        c.set(Calendar.HOUR_OF_DAY, 00);
        c.set(Calendar.MINUTE, 00);
        c.set(Calendar.SECOND, 00);
        c.set(Calendar.MILLISECOND, 00);
        return c.getTime();
    }

    /**
     * 获得日期结束时间 格式为：xxxx-yy-zz hh24:mi:ss (eg: 2007-06-22 23:59:59)
     *
     * @param d 日期
     * @return 日期
     */
    public static Date getEndDate(Date d) {
        Calendar c = Calendar.getInstance();
        if (d != null) {
            c.setTime(d);
        }

        c.set(Calendar.HOUR_OF_DAY, 23);
        c.set(Calendar.MINUTE, 59);
        c.set(Calendar.SECOND, 59);
        return c.getTime();
    }

    /**
     * 格式化标准日期和时间
     *
     * @param date 日期
     * @return 日期
     */
    public static String formatStandardTime(Date date, String pattern) {
        if (date == null) {
            return "";
        }
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        return sdf.format(date);
    }

    /**
     * 比较日期大小(fistDate > secondDate返回true)
     * @param firstDate
     * @param secondDate
     * @return
     */
    public static boolean compareDate(Date firstDate, Date secondDate) {
        Calendar cal1 = Calendar.getInstance();
        Calendar cal2 = Calendar.getInstance();
        try {
            cal1.setTime(new SimpleDateFormat("yyyy-MM-dd").parse(formatStandardTime(firstDate, "yyyy-MM-dd")));
            cal2.setTime(new SimpleDateFormat("yyyy-MM-dd").parse(formatStandardTime(secondDate, "yyyy-MM-dd")));
        } catch (ParseException e) {
            logger.error(e.getMessage());
        }
        return cal1.getTime().compareTo(cal2.getTime()) >= 0;
    }

    public static boolean sameDate(Date d1, Date d2) {
        SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd");
        //fmt.setTimeZone(new TimeZone()); // 如果需要设置时间区域，可以在这里设置
        return fmt.format(d1).equals(fmt.format(d2));

    }

    public static Date getFirstDayOfWeek() {
        Date date = new Date();
        Calendar cal = Calendar.getInstance();
        try {
            cal.setTime(date);
            cal.set(Calendar.DAY_OF_WEEK, 2);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return cal.getTime();
    }

    public static Date getFirstDayOfMonth() {
        Calendar cale = Calendar.getInstance();
        cale.add(Calendar.MONTH, 0);
        cale.set(Calendar.DAY_OF_MONTH, 1);
        return cale.getTime();
    }

    public static Date getFirstDayOfYear(){
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.YEAR,0);
        calendar.set(Calendar.DAY_OF_YEAR,1);
        return calendar.getTime();
    }
}
