package cn.aiyangniu.api.service.impl.system;

import cn.aiyangniu.api.service.system.SysTypeService;
import cn.aiyangniu.api.common.entity.system.SysTypeEntity;
import cn.aiyangniu.api.common.entity.system.SysTypeVo;
import cn.aiyangniu.api.mapper.system.SysTypeMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * SysTypeServiceImpl 分类服务实现类
 *
 * @author Henry Hall
 * @since 2020-08-08
 */
@Service
public class SysTypeServiceImpl implements SysTypeService {

    @Resource
    private SysTypeMapper typeMapper;

    /**
     * addType 添加分类
     *
     * @param typeEntity 分类实体对象
     * @return 返回保存结果
     */
    @Override
    public int addType(SysTypeEntity typeEntity) {
        return typeMapper.addType(typeEntity);
    }

    /**
     * delTypes 删除分类
     *
     * @param sysTypeVo 分类Vo对象
     * @return 返回删除结果
     */
    @Override
    public int delTypes(SysTypeVo sysTypeVo) {
        List<String> idList = new ArrayList<>();
        List<SysTypeEntity> typeEntities = typeMapper.selTypes(sysTypeVo);
        for(String s : sysTypeVo.getTypeIds()) {
            getTypeIds(typeEntities, s, idList);
        }
        sysTypeVo.setTypeIds(idList);
        return typeMapper.delTypes(sysTypeVo);
    }

    /**
     * getType 获取单一分类
     *
     * @param typeId 分类编号
     * @return 返回分类对象
     */
    @Override
    public SysTypeEntity getType(String typeId) {
        return typeMapper.getType(typeId);
    }

    /**
     * edtType 修改分类
     *
     * @param typeEntity 分类实体对象
     * @return 返回修改结果
     */
    @Override
    public int edtType(SysTypeEntity typeEntity) {
        return typeMapper.edtType(typeEntity);
    }

    /**
     * listTypes 列表显示分类
     *
     * @param kind 分类种类编号
     * @return 返回分类列表
     */
    @Override
    public List<SysTypeEntity> listTypes(String kind, String parentId) {
        List<SysTypeEntity> newList = null;
        List<SysTypeEntity> typeEntityList = typeMapper.listTypes(kind);
        // 用于重新排序展示
        if(typeEntityList!=null && typeEntityList.size()>0) {
            newList = new ArrayList<>();
            int modLevel = 0;
            for(SysTypeEntity curEnt: typeEntityList) {
                if(parentId.equals(curEnt.getParentId())) {
                    curEnt.setTypeLevel(modLevel);
                    newList.add(curEnt);
                    getChild(typeEntityList, curEnt.getTypeId(), newList, modLevel);
                }
            }
        }
        return newList;
    }

    /**
     * sortTypes 统计子级分类数量，用于排序
     *
     * @param parentId 请求参数
     * @return 返回新增结果
     */
    @Override
    public int sortTypes(String parentId) {
        return typeMapper.sortTypes(parentId);
    }

    /**
     * selTypes 下拉选择展示分类
     *
     * @param sysTypeVo 分类Vo对象
     * @return 返回新增结果
     */
    @Override
    public List<SysTypeEntity> selTypes(SysTypeVo sysTypeVo) {
        List<SysTypeEntity> typeEntityList = typeMapper.selTypes(sysTypeVo);
        if(typeEntityList!=null && typeEntityList.size()>0) {
            typeEntityList = buildTrees(typeEntityList, sysTypeVo.getParentId());
        }
        return typeEntityList;
    }

    /**
     * getTypeIds 获取树型的节点下所有子节点编号
     *
     * @param ents 模块的List
     * @param cid  当前模块编号
     */
    private void getTypeIds(List<SysTypeEntity> ents, String cid, List<String> idList) {
        if(ents != null && ents.size() > 0 && !"".equals(cid)) {
            idList.add(cid);
            List<SysTypeEntity> list = getTypes(ents, cid);
            if(list.size() > 0) {
                for(SysTypeEntity cEntity : list) {
                    getTypeIds(ents, cEntity.getTypeId(), idList);
                }
            }
        }
    }

    /**
     * getTypes 获取树型的节点下所有子节点对象List
     *
     * @param ents 模块的List
     * @param cid  当前模块编号
     */
    private List<SysTypeEntity> getTypes(List<SysTypeEntity> ents, String cid) {
        List<SysTypeEntity> list = new ArrayList<>();
        if(ents != null && ents.size() > 0) {
            for(SysTypeEntity cEntity : ents) {
                if(cid.equals(cEntity.getParentId())) {
                    list.add(cEntity);
                }
            }
        }
        return list;
    }

    /**
     * buildTrees 递归筛选组织前端所需要的数据格式
     *
     * @param lists    组织对象数组List
     * @param parentId 父级编号
     * @return 返回要求的数据格式
     */
    private List<SysTypeEntity> buildTrees(List<SysTypeEntity> lists, String parentId) {
        List<SysTypeEntity> entities = new ArrayList<>();
        if(lists != null && lists.size() > 0) {
            for(SysTypeEntity cEntity : lists) {
                if(cEntity.getParentId().equals(parentId)) {
                    entities.add(findChildren(cEntity, lists));
                }
            }
        }
        return entities;
    }

    private static SysTypeEntity findChildren(SysTypeEntity typeEntity, List<SysTypeEntity> lists) {
        if(lists != null && lists.size() > 0) {
            for(SysTypeEntity entity : lists) {
                if(entity.getParentId().equals(typeEntity.getTypeId())) {
                    if(typeEntity.getChildren() == null) {
                        typeEntity.setChildren(new ArrayList<>());
                    }
                    typeEntity.getChildren().add(findChildren(entity, lists));
                }
            }
        }
        return typeEntity;
    }

    /**
     * 从已有List中找出与父级编号一致的，放到新的List中, 用于递归排序
     *
     * @param lists 已有List数组
     * @param parentId 父级编号
     * @param newList 新的List数组
     * @param level 模块级别
     */
    private static void getChild(List<SysTypeEntity> lists, String parentId, List<SysTypeEntity> newList, int level) {
        if(lists != null && lists.size() > 0) {
            level++;
            for(SysTypeEntity entity : lists) {
                if(entity.getParentId().equals(parentId)) {
                    entity.setTypeLevel(level);
                    newList.add(entity);
                    getChild(lists, entity.getTypeId(), newList, level);
                }
            }
        }
    }

    /**
     * getParentTypes 向上递归查询所有父节点（每一层的父节点只有一个）
     * @param id 当前节点编号
     * @param entityList 实体对象集合
     * @return 返回递归结果
     */
    private List<SysTypeEntity> getParentTypes(String id, List<SysTypeEntity> entityList) {
        // 向上递归查询所有父节点
        SysTypeEntity typeEntity = typeMapper.getType(id);
        if(typeEntity != null) {
            entityList.add(typeEntity);
            getParentTypes(typeEntity.getTypeId(), entityList);
        }
        return entityList;
    }
}
